==> rakuten_spiders-0.0.1-py3-none-any.whl: rakuten/spiders/product.py <==
import json

import scrapy
from rakuten.product_manager.formatter import ProductParser
from scrapy.crawler import CrawlerProcess
from scrapy.utils.project import get_project_settings


class ProductSpider(scrapy.Spider):
    name = "product"

    custom_settings = {
        "ITEM_PIPELINES": {
            "rakuten.pipelines.ProductPipeline": 400,
        }
    }
    cookies = {
        "_ra": "1720136799401|4eaf8b28-a45e-4440-b863-a7dce12f0943",
        "Rp": "7c3d8621347851b825922d26b56687345f4d3cc",
        "ak_bmsc": "9D6C9AA7824F4AF87382EF16C9F4BCAC~000000000000000000000000000000~YAAQxmPKFwZFTDSSAQAATqZXRhkMsLA8FO6LaOwBoi86FiuxayXNATUkfircCPanwIZEeT5RSZQGf8S/L0+TNYdqILMHMO4ylenugUoutiniFNWPV70IPAbYBlX9pcTjHiPFcrMFljl3mZNOTvDX9IrOpnAB9eYlH9SbzIYydcq8kT4jw/TKo+jgirjKss2d52FeU2KUCVfiiKGAEXHje84YIt81rMy5nHW/ssEePaCgK/Q6iNujU7qk4TbM11C38T/TRVQ/TQ3Q/9JYrB4IATT4c2N84beCzu0ArPIkywYXQBHyr9hu3M2yclish/JjmUeReFNmSjJnSrlRpfbdp85c1Zl/jlngdzXfoYziq3IUDkMqRYWv7zDVzb8MAtCl5bk3pnyBzyPIW2naUUQ6nK1Vq4q2VD7qMOoXxicVnsB/3j7DakF0/DYSV+vwaZx/Fi8=",
        "rat_v": "4512a3c62812b2fd344dd37d47c66fb7aca5bf7b",
        "bm_sv": "00D8C750387FB9F52BEDF13562B9FBA8~YAAQxmPKFztFTDSSAQAARqtXRhlcYAejP+59gt4+GSCTm79YyKr7Nrj8kDv9/DFc0K6y5C5OOGOa0D34O32h29+9xrGkyXIwj7QJH2sqb4PnoVVPSb9guVdG8HRd+325te67P5LNiM/gmqSkJvbJKG3aIJpiSTzW50Y+d1dseRIuPVk9eYUO3Jpv/2G+S5TpjcEafn2FhSEuSB9ywxcoLy+BGksJJF8bVRQ0i+k4RHLvSOwvIn+T2HG9aObJxgmhcgbe~1",
        "__lt__cid.3df24f5b": "119d419e-5897-4080-a946-aaf888c2f174",
        "__lt__sid.3df24f5b": "b976b54b-2859f6db",
        "_gcl_au": "1.1.1839787249.1727757010",
        "cto_bundle": "2np6BV9KRmxoZlNWc21VV081azclMkZ6UWVhT3QlMkJGcWNBQ1VtT0xZVGpCSUJRTFhJSUI3Q3ZWZm1uVXFEenRrMHBxJTJGUWQ2dlIyRFg4cnJUWVdYSjZnSGZEM3RTU1VBWTlTR2lieURXY004QXJlczFta0QwS1k5eGlwOCUyRmolMkZqd2hXQUdXTDhmeW1DVU8lMkZhbGxzVjBmWFh3b0hVeHclM0QlM0Q",
        "_uetsid": "d807bc407fad11ef950669d0b76826bf",
        "_uetvid": "ae615ba03a5f11efacd9217195dc650b",
    }

    headers = {
        "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
        "accept-language": "en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7",
        "cache-control": "no-cache",
        "pragma": "no-cache",
        "priority": "u=0, i",
        "sec-ch-ua": '"Chromium";v="124", "Google Chrome";v="124", "Not-A.Brand";v="99"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Linux"',
        "sec-fetch-dest": "document",
        "sec-fetch-mode": "navigate",
        "sec-fetch-site": "none",
        "sec-fetch-user": "?1",
        "upgrade-insecure-requests": "1",
        "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36",
    }

    def start_requests(self):
        urls = [
            "https://www.rakuten.co.jp/category/201541/?l-id=www_PC_category/medicine_gt_mdc_pc_leftnavi",
            "https://www.rakuten.co.jp/category/568561/?l-id=www_PC_category/medicine_gt_mdc_pc_leftnavi",
            "https://www.rakuten.co.jp/category/201485/?l-id=www_PC_category/health_gt_hlt_pc_leftnavi",
            "https://www.rakuten.co.jp/category/567602/",
            "https://www.rakuten.co.jp/category/beauty/?l-id=top_normal_gmenu_d_beauty_others",
        ]
        for url in urls:
            yield scrapy.Request(url, headers=self.headers)

    def get_category_urls(self, response):
        return response.css(".dui-card>div.title>a::attr(href)").getall()

    def get_main_category_urls(self, response):
        return response.css("a.gtc-searchGenreCard__itemLink.gtu-link::attr(href)").getall()

    def parse(self, response):
        cats_url = self.get_category_urls(response)
        main_cats = self.get_main_category_urls(response)
        if cats_url:
            for cat_url in cats_url:
                yield scrapy.Request(url=cat_url, headers=self.headers, callback=self.parse_listing)

        if main_cats:
            for cat_url in main_cats:
                yield scrapy.Request(url=cat_url, headers=self.headers, callback=self.parse)

        yield scrapy.Request(
            url=response.url,
            headers=self.headers,
            callback=self.parse_listing,
            dont_filter=True,
        )

    def parse_listing(self, response):
        category = response.css("span.item.-active::text").get()
        try:
            json_data = json.loads(
                response.css("script").re_first("window.__INITIAL_STATE__ = (.+);")
            )
        except Exception as e:
            return None

        for product_url in (
            json_data.get("state", {}).get("data", {}).get("ichibaSearch", {}).get("items", [])
        ):
            yield scrapy.Request(
                url=product_url.get("url"),
                headers=self.headers,
                callback=self.parse_product,
                meta={
                    "category": category,
                    "sold": product_url.get("isSoldOut"),
                    "reviews": product_url.get("review", {}).get("numReviews"),
                    "rating": product_url.get("review", {}).get("score"),
                },
            )

        next_page_url = response.css("a.nextPage::attr(href)").get()
        if next_page_url:
            yield scrapy.Request(
                url=next_page_url, headers=self.headers, callback=self.parse_listing
            )

    def parse_product(self, response):
        self.logger.info(f"{response.url}: {response.status}")
        try:
            product_parser = ProductParser()
            product = product_parser.parse(response)
            if product:
                yield product
        except Exception as e:
            print(e)


def main():
    process = CrawlerProcess(get_project_settings())
    process.crawl(ProductSpider)
    process.start()


if __name__ == "__main__":
    main()

==> rakuten_spiders-0.0.6-py3-none-any.whl: rakuten/spiders/product.py <==
import json

import scrapy
from rakuten.product_manager.formatter import ProductParser
from scrapy.crawler import CrawlerProcess
from scrapy.utils.project import get_project_settings


class ProductSpider(scrapy.Spider):
    name = "product"

    custom_settings = {
        "ITEM_PIPELINES": {
            "rakuten.pipelines.ProductPipeline": 400,
        }
    }

    cookies = {
        "_ra": "1723913689980|e921b9b0-fa0e-4b72-b15c-ffa032265483",
        "Rp": "fc7f7af38e8d9f27367ba1a3e866c0d5da1c715",
        "Apache": "74c610a.61fe3ef555522",
        "s_pers": "%20s_mrcr%3D1100400000000000%257C4000000000000%257C4000000000000%257C4000000000000%7C1881593694589%3B",
        "__lt__cid.3df24f5b": "bbcba912-773c-4f0d-b884-3bbe80651a63",
        "_yjsu_yjad": "1723913735.5a8f8613-9019-4988-a590-3a111217bbb2",
        "_gcl_au": "1.1.1872737652.1723913735",
        "_tt_enable_cookie": "1",
        "_ttp": "tNbkXM3N2yU-gWj7wOQisW5GRCk",
        "sui_hide_shipping": "20230316",
        "cto_bundle": "tZ4MdF9CWXNTTlZZbUdkOGMwSlZLaSUyRlpXajlnQXBnREN4c2Y0d0lZQWg5S1pWbHE3cCUyQnlJcUd0VGtDJTJGJTJGWmtKYVRKV0RJZnlSazBPTkU2QmhBdUJLZ3A4QmlVMEZwb0lRem9TdW1DWmF2WEs1bXRvY2IlMkJQaEolMkI0SzJnbW5GSjdGOTEyUFI1WGklMkJFZWdiQzlQSmF3TDY5NFh3VGpRQlNHdmY1WjBoaGtmcVZlT0Y3UXF6UjNrYlg3aiUyRmZPUHBjM3FaM3B2UGJiS2JQN2YyTGlhQk9jZmhsJTJCSjljYU5BUnRSWVV3NVNXZTdzdDI2ZllDdUxCTWxnMFFBaU8lMkJ0eTFBZU93a24",
        "_uetvid": "6303d830324511efbaaab713d51cdb8b",
        "Re": "31.4.4.0.0.510103.7:33.3.1.0.0.568569.1:31.3.1.0.0.503054.1:31.10.2.0.0.567540.2:31.2.17.0.0.216600.2:33.3.3.0.0.568571.3:33.2.1.15.2.553397.1:33.2.2.6.0.553409.1:31.3.3.0.0.216496.1:31.3.2.0.0.216495.1:31.6.0.0.0.111120.5:32.2.1.5.0.302651.1:33.2.1.6.2.553361.1:31.1.13.0.0.503044.2-31.4.4.0.0.510103.7:33.3.1.0.0.568569.1:31.3.1.0.0.503054.1:31.10.2.0.0.567540.2:31.2.17.0.0.216600.2:33.3.3.0.0.568571.3:33.2.1.15.2.553397.1:33.2.2.6.0.553409.1:31.3.3.0.0.216496.1:31.3.2.0.0.216495.1:31.6.0.0.0.111120.5:32.2.1.5.0.302651.1:33.2.1.6.2.553361.1:31.1.13.0.0.503044.2",
        "ak_bmsc": "B50C1F0CB8A965B30B142807F5B5DEE5~000000000000000000000000000000~YAAQOh/XF6ttWHCSAQAAD+YZfRl0bze1nKMOZdFNfv+llAIGCuXwQBk1l91X2c4kbcoIQ2IgVFvAHirZ2jEaFx9QC3EKWzm9CFI06KTgovq4ak6U3n4groKFKD3fxa3iR0QWB5TnIudAM4A5C9Mb/QTInHDpFy7Mhtd8R/ohxWqU+WEKP8y3OWFS5cQY4I0Iq6OxBHxJaqX6LphZYyQRw060saqMX8tSYcRPDgMrwMNfhHiFc66KxEnxZEhl2HLUhcA//ocYMX0b39FwVNnTbwt1Amh8y+7P4QsyFgCG/s0ByWiaSGRzKkwXb8PbvtuJ+voo9PE8xn962hlzfcxEWoWcSoruIzW9S8J3fyW/gA4qUtKUxUQ9qRAKAbFZvrMMAkoGk9OOUrGocf0kNGzKjS2u09wr84NhZ7VRF0Pw59AF2Gtmpw1EkFcI",
    }

    headers = {
        "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
        "accept-language": "en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7",
        "cache-control": "no-cache",
        "pragma": "no-cache",
        "priority": "u=0, i",
        "sec-ch-ua": '"Chromium";v="124", "Google Chrome";v="124", "Not-A.Brand";v="99"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Linux"',
        "sec-fetch-dest": "document",
        "sec-fetch-mode": "navigate",
        "sec-fetch-site": "none",
        "sec-fetch-user": "?1",
        "upgrade-insecure-requests": "1",
        "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36",
    }

    def start_requests(self):
        urls = [
            "https://www.rakuten.co.jp/category/beauty/?l-id=top_normal_gmenu_d_beauty_others",
            "https://www.rakuten.co.jp/category/201541/?l-id=www_PC_category/medicine_gt_mdc_pc_leftnavi",
            "https://www.rakuten.co.jp/category/568561/?l-id=www_PC_category/medicine_gt_mdc_pc_leftnavi",
            "https://www.rakuten.co.jp/category/201485/?l-id=www_PC_category/health_gt_hlt_pc_leftnavi",
            "https://www.rakuten.co.jp/category/567602/",
        ]
        for url in urls:
            yield scrapy.Request(url, headers=self.headers)

    def get_category_urls(self, response):
        return response.css(".dui-card>div.title>a::attr(href)").getall()

    def get_main_category_urls(self, response):
        return response.css("a.gtc-searchGenreCard__itemLink::attr(href)").getall()

    def parse(self, response):
        cats_url = self.get_category_urls(response)
        main_cats = self.get_main_category_urls(response)
        if cats_url:
            for cat_url in cats_url:
                yield scrapy.Request(url=cat_url, headers=self.headers, callback=self.parse_listing)

        if main_cats:
            for cat_url in main_cats:
                yield scrapy.Request(url=cat_url, headers=self.headers, callback=self.parse)

        yield scrapy.Request(
            url=response.url,
            headers=self.headers,
            callback=self.parse_listing,
            dont_filter=True,
        )

    def parse_listing(self, response):
        category = response.css("span.item.-active::text").get()
        try:
            json_data = json.loads(
                response.css("script").re_first("window.__INITIAL_STATE__ = (.+);")
            )
        except Exception as e:
            return None

        for product_url in (
            json_data.get("state", {}).get("data", {}).get("ichibaSearch", {}).get("items", [])
        ):
            yield scrapy.Request(
                url=product_url.get("url"),
                headers=self.headers,
                callback=self.parse_product,
                meta={
                    "category": category,
                    "sold": product_url.get("isSoldOut"),
                    "reviews": product_url.get("review", {}).get("numReviews"),
                    "rating": product_url.get("review", {}).get("score"),
                },
            )

        next_page_url = response.css("a.nextPage::attr(href)").get()
        if next_page_url:
            yield scrapy.Request(
                url=next_page_url, headers=self.headers, callback=self.parse_listing
            )

    def parse_product(self, response):
        self.logger.info(f"{response.url}: {response.status}")
        try:
            product_parser = ProductParser()
            product = product_parser.parse(response)
            if product:
                yield product
        except Exception as e:
            print(e)


def main():
    process = CrawlerProcess(get_project_settings())
    process.crawl(ProductSpider)
    process.start()


if __name__ == "__main__":
    main()
